#!/usr/bin/env python
"""A script to scrape items from an Amazon wishlist. The script only works for
wishlists which are "Public". You can change the settings by following the
instruction in:
http://www.amazon.com/gp/help/customer/display.html?nodeId=501094

    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from collections import namedtuple
import logging
import os
import sys
from urllib.parse import urlparse

import requests
from lxml import html

from sornobase import loggingutil
from sornobase import consoleutil


_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()
_PLAIN_ERROR_LOGGER = None  # will be created in main()

Item = namedtuple('Item', 'id title url')


class App(object):
    WISHLIST_PAGE_TEMPLATE = (
        "https://www.amazon.com/gp/registry/wishlist"
        + "/{wishlist_id}/?page={page_number}"
    )
    HEADERS = {
        'user-agent': 'Mozilla/5.0 ;Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; like Gecko',
    }

    def __init__(self, wishlist_id):
        self.wishlist_id = wishlist_id

    def run(self):
        # stores the id's of items
        seen_items = set()

        page_number = 1
        item_number = 1
        num_of_empty_page_reached = 0

        while True:
            items = self.get_items_from_page_num(page_number)

            rows = []
            for item in items:
                if item.id in seen_items:
                    _LOG.debug("Seen title %s, skip it", item.title)
                else:
                    seen_items.add(item.id)
                    rows.append(
                        {
                            'no.': str(item_number),
                            'title': item.title,
                            'url': item.url if item.url else "",
                        }
                    )
                    item_number += 1
                    num_of_empty_page_reached = 0

            if not rows:
                num_of_empty_page_reached += 1
                if num_of_empty_page_reached >= 3:
                    # All items are seen in the fetch, so we are done
                    # Sometimes amazon returns 0 items even we havn't reached
                    # to the end, so give it a few trials
                    break
                else:
                    continue

            data_printer = consoleutil.DataPrinter(
                rows,
                headers=('no.', 'title', 'url'),
                delimiter='\t',
                print_func=_PLAIN_LOGGER.info,
            )

            data_printer.print_result(
                style=consoleutil.DataPrinter.PRINT_STYLE_PLAIN
            )

            page_number += 1

    def get_items_from_page_num(self, num):
        url = self.WISHLIST_PAGE_TEMPLATE.format(
            wishlist_id=self.wishlist_id,
            page_number=num,
        )
        _LOG.debug("Fetch from: %s", url)

        wishlist_page = requests.get(url)
        wishlist_page_html = wishlist_page.text
        _PLAIN_ERROR_LOGGER.debug(wishlist_page_html)

        tree = html.fromstring(wishlist_page_html)
        all_h5_nodes = tree.xpath("//div[@class='a-row a-size-small']/h5")

        items = []
        for h5_node in all_h5_nodes:
            try:
                item = self._get_item_from_idea_h5_node(h5_node)
                if not item:
                    item = self._get_item_from_amazon_item_h5_node(h5_node)

                if item:
                    items.append(item)
                else:
                    _LOG.warn("Fail to retrieve an item for snippet")
                    _PLAIN_ERROR_LOGGER.warn("===== Start of snippet =====")
                    _PLAIN_ERROR_LOGGER.warn(html.tostring(h5_node))
                    _PLAIN_ERROR_LOGGER.warn("===== End of snippet =====")
            except ValueError as ex:
                _LOG.exception("Fail to retrieve an item: %s", ex)
                _PLAIN_ERROR_LOGGER.warn("===== Start of snippet =====")
                _PLAIN_ERROR_LOGGER.warn(html.tostring(h5_node))
                _PLAIN_ERROR_LOGGER.warn("===== End of snippet =====")

        return items

    def _get_item_from_idea_h5_node(self, h5_node):
        """
        Gets the item in a H5 html node that contains an Idea. Returns
        None if an Idea cannot be found.

        The H5 html node supposes to be like the following, "{param}" denotes
        the parameters of the item:

        <h5>
        ...
        <span id="itemName_{item id}">{item title}</span>
        ...
        </h5>
        """
        span_nodes = h5_node.xpath(
            ".//span[contains(@id, 'itemName_')]"
        )

        if not span_nodes:
            return None

        span_node = span_nodes[0]
        item_title = self.get_text_from_element(span_node)
        item_id = span_node.attrib['id'].split('itemName_')[1]
        return Item(id=item_id, title=item_title, url=None)

    def _get_item_from_amazon_item_h5_node(self, h5_node):
        """
        Gets the item in a H5 html node that contains an Amazon item. Returns
        None if an Amazon item cannot be found. An Amazon item is an item in
        wishlish that is sold in Amazon.

        The H5 html node supposes to be like the following, "{param}" denotes
        the parameters of the item:

        <h5>
        ...
        <a id="itemName_{item id}" href="{item url}">{item title}</a>
        ...
        </h5>
        """
        anchor_nodes = h5_node.xpath(".//a[contains(@id, 'itemName_')]")
        if anchor_nodes:
            # This is an Amazon item node
            anchor_node = anchor_nodes[0]

            item_url = "http://www.amazon.com" + anchor_node.attrib['href']
            item_title = self.get_text_from_element(anchor_node).strip()
            item_id = anchor_node.attrib['id'].split('itemName_')[1]
            return Item(id=item_id, title=item_title, url=item_url)

        return None

    def same_item_lists(self, prev_items, items):
        if prev_items is None or len(prev_items) != len(items):
            return False

        for prev, cur in zip(prev_items, items):
            prev_query = urlparse(prev.attrib['href']).query
            cur_query = urlparse(cur.attrib['href']).query
            if prev_query != cur_query:
                return False

        return True

    def get_text_from_element(self, node):
        """
        Return a plain text representation of an html node.
        """
        text_segments = []
        self._collect_text_from_element(node, text_segments)
        return "".join(text_segments)

    def _collect_text_from_element(self, node, text_segments):
        """
        Collect text from node and all its children recursively and put into
        text_segments as a list of strings.
        """
        if node.tag.lower() == "br":
            text_segments.append(os.linesep)

        if node.text:
            text_segments.append(node.text)

        for child in node:
            self._collect_text_from_element(child, text_segments)

        if node.tail:
            text_segments.append(node.tail)


def parse_args(cmd_args):
    description = """
        A script to scrape items from an Amazon wishlist. The script only
        works for wishlists which are "Public". You can change the settings by
        following the instruction in:
        http://www.amazon.com/gp/help/customer/display.html?nodeId=501094
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "wishlist_id",
        help="When you look at the URL of your wishlist, it's something like"
        + " https://www.amazon.com/gp/registry/wishlist/<wishlist id>/ref=cm_wl_list_o_0?"
        + ", so just copy the wishlist id for this argument",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER, _PLAIN_ERROR_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")
    _PLAIN_ERROR_LOGGER = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        stdout=False,
    )

    app = App(args.wishlist_id)
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Downloads podcasts given a feed url.

The downloaded podcasts have useful file names (e.g contain the title of the
podcast and prefixed by the published date).


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import re
import sys

import dateutil.parser as dateutil_parser
import feedparser
import requests
from sornobase import loggingutil
from sornobase import webutil
import tzlocal


_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


class PodcastDownloader(object):
    def __init__(
        self,
        feed_url,
        out_dir=".",
    ):
        self.feed_url = feed_url
        self.out_dir = out_dir

    def run(self):
        _LOG.info("Find podcasts from %s", self.feed_url)
        parsed = feedparser.parse(self.feed_url)
        entries = parsed['entries']

        num_entries = len(entries)
        for i, entry in enumerate(entries):
            published = entry['published']
            link = entry['link']
            title = entry['title']

            _LOG.info(
                "(%s/%s) Download podcast %s from %s (published at: %s)",
                i + 1,
                num_entries,
                title,
                link,
                published,
            )

            dt = dateutil_parser.parse(published)
            local_dt = dt.astimezone(tzlocal.get_localzone())
            _, ext = os.path.splitext(link)

            new_filename = "{date} {title}{ext}".format(
                date=local_dt.strftime("%Y_%m_%d"),
                title=re.sub(
                    '  +',
                    ' ',
                    re.sub('[^-_a-zA-Z0-9()]', ' ', title),
                ),
                ext=ext,
            )
            new_filepath = os.path.join(self.out_dir, new_filename)
            tmp_new_filepath = new_filepath + ".tmp"

            _LOG.info(
                "Published date in local time: %s, download to file: [%s]",
                local_dt.strftime("%Y/%m/%d %H:%M:%S"),
                tmp_new_filepath,
            )
            if os.path.exists(new_filepath):
                _LOG.info("[%s] already exists, skipped", new_filepath)
                continue

            webutil.download_file(link, tmp_new_filepath)
            _LOG.info("Rename [%s] to [%s]", tmp_new_filepath, new_filepath)
            os.rename(tmp_new_filepath, new_filepath)


def parse_args(cmd_args):
    description = """
        Download podcasts from a feed with appropriate file names. This tool
        find all podcasts from a feed and download them to a directory
        specified by --out_dir. Each file name is consisted of the date in
        local timezone for the published date of the podcast, and the title of
        the podcast.
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--out-dir",
        default=".",
        help="The directory to store the podcasts. Default to \"%(default)s\"",
    )
    parser.add_argument(
        "feed_url",
        help="URL of a feed, should point to an xml file",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")

    app = PodcastDownloader(args.feed_url, out_dir=args.out_dir)
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Extracts Simon Property Group property information from its 10-K filings

Sample usage::

    $ sorno_extract_spg_properties.py spg_10-k.html

If you get UnicodeEncodingError, you should prefix your command with
"PYTHONIOENCODING=UTF-8". E.g::

    $ PYTHONIOENCODING=UTF-8 sorno_extract_spg_properties.py html_file


    Copyright 2017 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

import re
import sys
import subprocess

from bs4 import BeautifulSoup

class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        soup = BeautifulSoup(open(self.args.html_file).read(), "lxml")
        tables = soup.find_all("table")
        property_tables = [t for t in tables if "Property Name" in self.compress_spaces(t.text)]
        for property_table in property_tables:
            self.print_property_table(property_table)
        return 0

    def print_property_table(self, property_table):
        for tr in property_table.find_all("tr"):
            print("\t".join([self.compress_spaces(td.text).strip() for td in tr.find_all("td")]))

    def compress_spaces(self, s):
        return re.sub(r"\s+", " ", s.replace(u"\xa0", " "))


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2017")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("html_file")

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Batch posting tweets on Twitter

Before using the script, go to
https://dev.twitter.com/oauth/overview/application-owner-access-tokens to get
the necessary credentials.

Use Google Doc to edit your tweets, one line per tweet. You should not use naked links (i.e. each link should be associated with some text). Then "File" -> "Download as" -> "Web Page (.html zipped)".

Unzip the downloaded file. Then run the following command with the appropriate parameters. path_to_file should be the path to the html file you unzipped.

	$ sorno_twitter_post_tweets.py --consumer-key consumer_key --consumer-secret consumer_secret --access_token-key access_token_key --access-token-secret access_token_secret --parse-tweets-from-file path_to_file

The script prints each tweet, and asks if you want to post the tweet indicated by "Tweet preview". Enter "y" if you want it posted, "n" otherwise.


    Copyright 2017 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from collections import namedtuple
import copy
import logging
import re
import subprocess
import sys
from urllib.parse import urlparse, parse_qs

import twitter
import bs4

from sornobase import consoleutil
from sornobase import loggingutil


_log = logging.getLogger()

Tweet = namedtuple('Tweet', ['text', 'links', 'original_node'])


class App(object):
    """A console application to do work"""
    def __init__(self, args):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args
        self.api = None

    def run(self):
        """The entry point of the script"""
        args = self.args
        _log.debug("%s", args)
        self.api = twitter.Api(
            consumer_key=args.consumer_key,
            consumer_secret=args.consumer_secret,
            access_token_key=args.access_token_key,
            access_token_secret=args.access_token_secret,
        )
        user = self.api.VerifyCredentials()
        if user is None:
            _log.error("Invalid user credentials")
            return 1

        if args.parse_tweets_from_file:
            with open(args.parse_tweets_from_file) as f:
                content = f.read()

            soup = bs4.BeautifulSoup(content, 'html.parser')
            self.batch_tweeting_with_soup(soup)

        _log.debug("user %s", user)
        return 0

    def batch_tweeting_with_soup(self, soup):
        tweets = []
        for p in soup.find_all('p'):
            tweet = self.make_tweet(p, remove_anchored_text=self.args.remove_anchored_text)
            if not tweet.text:
                continue
            tweets.append(tweet)

        for i, tweet in enumerate(tweets):
            print("Tweet %s:" % (i + 1))
            self.post_tweet(tweet)
            print("\n" * 2)

    def make_tweet(self, node, remove_anchored_text=False):
        anchors = node.find_all('a')
        links = [self.clean_link(a['href']) for a in anchors]
        text = self.get_text(node, remove_anchored_text=remove_anchored_text)
        return Tweet(text=text, links=links, original_node=node)

    def get_text(self, node, remove_anchored_text=False):
        if remove_anchored_text:
            node = copy.copy(node)
            for anchor in node('a'):
                anchor.replaceWith("")

        return re.sub(r"\s+", " ", node.text.replace('\xa0', ' ')).strip()

    def clean_link(self, link):
        """
        Clean a link from something like:
        https://www.google.com/url?q=https://blahblah.com&sa=D&ust=1498545552302000&usg=AFQjCNE8sJX0-SzPOwH545Ek44d3gNXtMg to https://blahblah.com
        """
        url = urlparse(link)
        params = {}
        if url.query:
            params = parse_qs(url.query)

        if url.netloc == "www.google.com" and url.path == "/url" and params.get('q'):
            return params.get('q')[0]

        return link

    def post_tweet(self, tweet):
        print("Text:", tweet.text)
        print("Links:", tweet.links)
        status = tweet.text
        status_len = len(status)
        for link in tweet.links:
            status = self.join_text(status, link)
        status_with_links_len = len(status)

        print("Tweet preview(%s,%s): %s" % (status_len, status_with_links_len, status))

        ans = consoleutil.input("\nTweet it[yes/no/remove-anchored-text]? ")
        ans = ans.lower()
        if ans in ("y", "yes"):
            result = self.api.PostUpdate(status)
            print("Tweeted:", result.text)
            print("Result:", result)
        elif ans in ("n", "no"):
            return
        elif ans in ("r", "remove", "remove-anchored-text"):
            print("\n*Edited*\n")
            tweet = self.make_tweet(tweet.original_node, remove_anchored_text=True)
            self.post_tweet(tweet)

    def join_text(self, text, more_text):
        if text == "":
            return more_text
        return text + " " + more_text

    def _run_cmd(self, cmd):
        """Run a shell command

        Args:
            cmd (string or a list of strings): The shell command to run. If
                it's a string, uses the system shell to run the command. If
                it's a list of strings, the first string is the program to run
                and the rest are its arguments. The arguments are quoted
                properly by the subprocess module, so the arguments do not
                have to be quoted when passing to this method.
        """
        _log.info(cmd)
        if isinstance(cmd, list):
            use_shell = False
        else:
            use_shell = True
        return subprocess.check_call(cmd, shell=use_shell)


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2017")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--consumer-key",
    )
    parser.add_argument(
        "--consumer-secret",
    )
    parser.add_argument(
        "--access-token-key",
    )
    parser.add_argument(
        "--access-token-secret",
    )
    parser.add_argument(
        "--parse-tweets-from-file",
    )
    parser.add_argument(
        "--remove-anchored-text",
        action="store_true",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import PyPDF2

def pdf_to_text(filepath):
    with open(filepath, "rb") as file_obj:
        pdf_reader = PyPDF2.PdfFileReader(file_obj)
        num_pages = pdf_reader.numPages

        count = 0
        text_segments = []
        while count < num_pages:
            text_segments.append(pdf_reader.getPage(count).extractText())
            count += 1

        return ''.join(text_segments)



#!/usr/bin/env python
"""sorno_grepchunks greps chunks for a regex.

Oftenly, you want to treat multiple lines as one chunk and see if it matches a
regex. If it does, you want to print out the whole chunk instead of the only
line that matches the regex. sorno_grepchunks lets you define what a chunk
is by giving a chunk starting regex, that is, all the lines starting from the
line that matches the regex and before the next match are treated as one
chunk. You can then apply another regex to match against it.

Examples:

    Setup:
        $ printf "chunk1\napple\nboy\nchunk2dog\n" > /tmp/sample

    The first chunk has a 'e', followed by a newline, then a 'b', so:
        $ sorno_grepchunks.py --chunk-starting-regex chunk "e.b" /tmp/sample
        chunk1
        apple
        boy

    Grep for "o" which appears in all two chunks:

        $ python scripts/sorno_grepchunks.py --chunk-starting-regex chunk "o" /tmp/sample
        chunk1
        apple
        boy
        chunk2dog

    Gre for "og" which appears only in the second chunk:
        $ python scripts/sorno_grepchunks.py --chunk-starting-regex chunk "og" /tmp/sample
        chunk2dog
        $


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import re
import sys

from sornobase import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class GrepChunksApp(object):
    """A console application to do work"""
    def __init__(
        self,
        args,
    ):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args
        self._partial_chunk = ""
        self._regex = re.compile(args.regex, re.DOTALL)

    def run(self):
        chunk_starting_regex = re.compile(self.args.chunk_starting_regex)
        for f in self.args.file:
            # Read the whole file and if a chunk is closed, grep it
            with open(f) as fh:
                for line in fh:
                    s = self._grep_if_exists(
                        self._feed(line, chunk_starting_regex)
                    )
                    if s is not None:
                        print(s, end="")

            s = self._grep_if_exists(self._close_partial_chunk())
            if s is not None:
                print(s, end="")

        return 0

    def _feed(self, line, chunk_starting_regex):
        chunk = None
        if chunk_starting_regex.search(line):
            chunk = self._close_partial_chunk()

        self._partial_chunk += line
        return chunk

    def _close_partial_chunk(self):
        if self._partial_chunk:
            chunk = self._partial_chunk
            self._partial_chunk = ""
            return chunk

        return None

    def _grep_if_exists(self, chunk):
        if chunk is not None:
            if self._regex.search(chunk):
                return chunk

        return None


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--chunk-starting-regex",
        help="The regex that matches the starting of a chunk. The ending of a"
        " chunk is defined as the end of a file or the line before the"
        " starting of another chunk."
    )
    parser.add_argument(
        "regex",
        help="Any chunks matching the regex are printed."
    )
    parser.add_argument(
        "file",
        nargs="+",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = GrepChunksApp(
        args,
    )
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_gcloud_bigquery.py demos the use of Google Cloud BigQuery

The script can be run to get the result of a query.

You need to get the json credentials file before using this script. See https://developers.google.com/identity/protocols/application-default-credentials#howtheywork.

Quickstart:

    sorno_gcloud_bigquery.py --google-json-credentials <your-json-credentials-file> "SELECT author,text FROM [bigquery-public-data:hacker_news.comments] where text is not null LIMIT 10"

Reference: https://cloud.google.com/bigquery/create-simple-app-api#authorizing

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import base64
import json
import logging
import os

import sys

import httplib2

from apiclient import discovery
from oauth2client import client as oauth2client
import googleapiclient

from sornobase import loggingutil
from sornobase import datetimeutil

BIGQUERY_SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
_log = logging.getLogger()


def create_bigquery_client(http=None):
    """
    https://developers.google.com/identity/protocols/application-default-credentials#howtheywork
    """
    credentials = oauth2client.GoogleCredentials.get_application_default()
    credentials = credentials.create_scoped(BIGQUERY_SCOPES)
    if not http:
        http = httplib2.Http()
    credentials.authorize(http)

    return discovery.build('bigquery', 'v2', http=http)

class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        if self.args.google_json_credentials:
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = (
                self.args.google_json_credentials
            )

        client = create_bigquery_client()
        project_id = self._get_project_id()

        query_request = client.jobs()
        query_data = {
            'query': self.args.query
        }
        query_response = query_request.query(
            projectId=project_id,
            body=query_data).execute()

        for row in query_response['rows']:
            print(row)

        return 0

    def _get_project_id(self):
        project_id = self.args.project_id
        if not project_id:
            # try to get it from credentials
            credentials_filename = oauth2client._get_environment_variable_file()
            if not credentials_filename:
                credentials_filename = oauth2client._get_well_known_file()

            if credentials_filename:
                with open(credentials_filename) as file_obj:
                    client_credentials = json.load(file_obj)
                    project_id = client_credentials['project_id']
        return project_id


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--google-json-credentials",
        help="Path to the google json credentials file. See https://developers.google.com/identity/protocols/application-default-credentials#howtheywork",
    )
    parser.add_argument(
        "--project-id",
    )
    parser.add_argument(
        "query",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])
    loggingutil.setup_logger(_log, debug=args.debug)

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Sends a simple email with plain text

The script first tries to use your system Mail Transfer Agent(MTA) configured,
otherwise, it prompts for login to use Gmail SMTP server.

The code is from
http://code4reference.com/2013/07/simple-python-script-to-send-an-email/


    Copyright 2015 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from email.mime.text import MIMEText
import getpass
import logging
import smtplib
import socket
import sys

from sornobase import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class App(object):
    def __init__(
        self,
        args,
    ):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args

    def run(self):
        """The entry point of the script
        """
        email = MIMEText(self.args.msg)
        email['Subject'] = self.args.subject
        email['From'] = self.args.from_address
        email['to'] = self.args.to_address

        try:
            self.send_email_with_local_mta(email)
        except socket.error as e:
            _log.error("Failed to use MTA to send the email: %s", e)
            _log.info("Try Gmail SMTP server instead...")
            self.send_email_with_gmail(email)
        return 0

    def send_email_with_local_mta(self, email):
        """Sends email with your system Mail Transfer Agent(MTA) configured

        Args:
            email(MIMEText): The email object to be sent.

        Raises:
            socket.error if cannot connect to an SMTP locally
        """
        server = smtplib.SMTP("localhost")
        server.sendmail(
            self.args.from_address,
            self.args.to_address,
            email.as_string(),
        )
        server.quit()

    def send_email_with_gmail(self, email):
        """Sends email with Gmail SMTP server

        The method prompts the user to get the login information for the Gmail
        SMTP server from the terminal.

        Args:
            email(MIMEText): The email object to be sent.
        """
        smtpserver = smtplib.SMTP("smtp.gmail.com", 587)
        smtpserver.ehlo()
        # Put SMTP connection in TLS mode and call ehlo again.
        smtpserver.starttls()
        smtpserver.ehlo()
        # Login to service
        password = getpass.getpass(
            "Enter your password for %s to use Gmail SMTP server:" % (
                self.args.from_address
            )
        )
        smtpserver.login(
            user=self.args.from_address,
            password=password,
        )
        # Send email
        smtpserver.sendmail(
            self.args.from_address,
            self.args.to_address,
            email.as_string(),
        )
        # close connection and session.
        smtpserver.quit();


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2015")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--from",
        dest="from_address",
    )
    parser.add_argument(
        "--to",
        dest="to_address",
    )
    parser.add_argument(
        "--subject",
    )
    parser.add_argument(
        "--msg",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = App(
        args,
    )
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Convert a PDF to text file

    Copyright 2017 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import PyPDF2

import argparse
import logging
import subprocess
import sys

from sornobase import loggingutil
from sorno import pdfutil

_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class App(object):
    """A console application to do work"""
    def __init__(self, args):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args

    def run(self):
        for pdf_file in self.args.pdf_file:
            print(pdfutil.pdf_to_text(pdf_file))
        return 0

    def _run_cmd(self, cmd):
        """Run a shell command

        Args:
            cmd (string or a list of strings): The shell command to run. If
                it's a string, uses the system shell to run the command. If
                it's a list of strings, the first string is the program to run
                and the rest are its arguments. The arguments are quoted
                properly by the subprocess module, so the arguments do not
                have to be quoted when passing to this method.
        """
        _log.info(cmd)
        if isinstance(cmd, list):
            use_shell = False
        else:
            use_shell = True
        return subprocess.check_call(cmd, shell=use_shell)


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2017")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "pdf_file",
        nargs='+',
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""A parser for a log that has a lot of java exceptions.

    Copyright 2017 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import subprocess
import sys

from sornobase import loggingutil


_log = logging.getLogger()


class App(object):
    """A console application to do work"""
    def __init__(self, args):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args

    def run(self):
        """The entry point of the script
        """
        with open(self.args.test_log, "rb") as f:
            self.parse_file(f)

        return 0

    def parse_file(self, test_log_file):
        for line in test_log_file:
            line = line.decode("utf-8", "ignore")
            if "Exception:" in line or "Error:" in line:
                print(line)

def parse_args(cmd_args):
    description = __doc__.split("Copyright 2017")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "test_log",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""A script that converts mulitple inputs of a java function to a single
Autovalue params class.

This script DOES NOT UNDERSTAND THE JAVA LANGUAGE. It simply uses some simple
parsing. Make sure you look at the result before using the code.

For the usages below, assume we have a class called MyConverter and the
following files:

# sample_function_defs.txt (you don't need the full body if you don't use
# --transform_func_def_with_autovalue option)

  String convertFunc(boolean f1, Optional<String> f2, Optional<Integer> f3, boolean f4) {
    print(f1);
    print(f2);
    print(f3);
    print(f4);
    return "nothing";
  }

# sample_function_defs_without_body.txt

  String convertFunc(boolean f1, Optional<String> f2, Optional<Integer> f3, boolean f4) {

# sample_convert_method_calls_in_test_file.txt (this can be the whole test file)

public void assert1() {
  assertThat(
      converter.convertFunc(
        /* f1= */ false,
        /* f2= */ Optional.empty(),
        Optional.of(123),
        /* f4= */ true)
      ).isEqualTo("nothing");
}

# sample_convert_call_site_file.txt (this should only contain the call site invocation)

converter.convertFunc(false, Optional.empty(), Optional.of(4), true);

# end of all files

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --output_autovalue_class

Output the definition of the autovalue class.
### begin of output
import com.google.auto.value.AutoValue;

/** Input parameters for convertFunc. */
@AutoValue
public abstract static class Params {

  public abstract boolean f1();

  public abstract Optional<String> f2();

  public abstract Optional<Integer> f3();

  public abstract boolean f4();

  public static Builder builder() {
      // TODO
      return new AutoValue_Params.Builder()
    .setF1(false)
    .setF4(false);
  }

  abstract Builder toBuilder();

  /** Builder for {@link Params} */
  @AutoValue.Builder
  public abstract static class Builder {

    public abstract Builder setF1(boolean f1);

    public abstract Builder setF2(Optional<String> f2);

    public abstract Builder setF3(Optional<Integer> f3);

    public abstract Builder setF4(boolean f4);

    public abstract Params build();
  }
}
### end of output

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --transform_func_def_with_autovalue

Transform the function definition to use the generated autovalue class.

### begin of output

  String convertFunc(Params params)) {
    print(params.f1());
    print(params.f2());
    print(params.f3());
    print(params.f4());
    return "nothing";
  }

### end of output

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --convert_method_calls_in_test_file sample_convert_method_calls_in_test_file.txt

Convert method calls in a test file to use the autovalue class.

### begin of output

public void assert1() {
  assertThat(
      converter.convertFunc(Params.builder()
.setF3(Optional.of(123))
.setF4(true)
.build())
      ).isEqualTo("nothing");
}

### end of output

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --convert_call_site_file sample_convert_call_site_file.txt

Convert the original call invocation to use the autovalue class.

### begin of output

converter.convertFunc(Params.builder()
.setF1(false)
.setF2(Optional.empty())
.setF3(Optional.of(4))
.setF4(true)
.build());

### end of output

    Copyright 2020 Heung Ming Tai
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

import re
import sys
import subprocess
from collections import namedtuple


fields = ('type', 'name', 'default_value')
Param = namedtuple('Param', fields, defaults=(None,) * len(fields))

AUTO_VALUE_CLASS_NAME = 'Params'

AUTO_VALUE_GET_METHOD_TEMPLATE = 'public abstract {type} {method_name}();'

AUTO_VALUE_SET_METHOD_TEMPLATE = 'public abstract Builder {method_name}({type} {param_name});'
AUTO_VALUE_CLASS_TEMPLATE = """\
import com.google.auto.value.AutoValue;

/** Input parameters for {func_name}. */
@AutoValue
public abstract static class {autovalue_class_name} {{
  {get_methods_defs}

  public static Builder builder() {{
      // TODO
      return new AutoValue_{autovalue_class_name}.Builder(){build_default_calls};
  }}

  abstract Builder toBuilder();

  /** Builder for {{@link {autovalue_class_name}}} */
  @AutoValue.Builder
  public abstract static class Builder {{
    {set_methods_defs}

    public abstract {autovalue_class_name} build();
  }}
}}
"""


class App(object):
    def __init__(self, args):
      self.args = args
      self.func_def_input_file = args.func_def_input_file

    def run(self):
      func_def_str = open(self.func_def_input_file).read()
      func_name = get_func_name(func_def_str)
      params = get_params(func_def_str)

      if self.args.output_autovalue_class:
        print_autovalue_class(self.get_autovalue_class_name(), func_name, params)

      if self.args.convert_method_calls_in_test_file:
        content = open(self.args.convert_method_calls_in_test_file).read()
        print(replace_content_for_test_file(self.get_autovalue_class_name(), func_name, params, content))

      if self.args.convert_call_site_file:
        content = open(self.args.convert_call_site_file).read()
        print(replace_content_for_call_site_file(self.get_autovalue_class_name(), func_name, params, content))

      if self.args.transform_func_def_with_autovalue:
        print(transform_func_def(self.get_autovalue_class_name(), func_name, params, func_def_str))

      return 0

    def get_autovalue_class_name(self):
      return self.args.autovalue_class_name


def get_func_name(s):
  """
  Gets the function name from a string.

  E.g. "void func(type f1)" -> "func"
  """
  pre_open_paren_str = s.split('(')[0]
  if ' ' not in pre_open_paren_str:
    return pre_open_paren_str

  return pre_open_paren_str[pre_open_paren_str.rindex(' ') + 1:]


def get_params(s):
  """
  Gets the list of input parameters (class Param) from a string.

  E.g. "void func(int f1, String f2)" -> [Param(...), Param(...)]
  """
  post_open_paren_str = s.split('(', 1)[1]
  params_str = get_until_close_paren(post_open_paren_str)
  params_strs = [s.strip() for s in params_str.split(',')]

  params = []
  for param_str in params_strs:
    # Need to prevent the case that the type and parameter names are separated
    # by newlines or more than one spaces, so replace all whitespaces to
    # single space.
    param_fragments = re.sub(r'\s+', ' ', param_str).split(' ')
    t, param_name = param_fragments[-2], param_fragments[-1]
    if t == 'Boolean':
      t = 'boolean'

    if t == 'boolean':
      param = Param(type=t, name=param_name, default_value='false')
    else:
      param = Param(type=t, name=param_name)

    params.append(param)

  return params


def get_until_close_paren(s):
  """
  Gets the string up to the closing parenthesis ')', bypassing all pairs
  of parentheses in-betwee.

  E.g. "false, (long) 123, 4).getSomething()" -> "false, (long) 123, 4"
  """
  open_paren_count = 0
  for i in range(len(s)):
    c = s[i]
    if c == ')':
      if open_paren_count:
        open_paren_count -= 1
      else:
        return s[:i]
    elif c == '(':
      open_paren_count += 1

  # no idea what to do, so just return s
  return s


def print_autovalue_class(autovalue_class_name, func_name, params):
  get_methods_defs_strs = []
  set_methods_defs_strs = []
  build_default_calls_strs = []
  for param in params:
    get_methods_defs_strs.append(AUTO_VALUE_GET_METHOD_TEMPLATE.format(
        type=param.type,
        method_name=param.name,
    ))

    set_method_name = 'set' + upperfirst(param.name)
    set_methods_defs_strs.append(AUTO_VALUE_SET_METHOD_TEMPLATE.format(
        method_name=set_method_name,
        type=param.type,
        param_name=param.name,
    ))

    if param.default_value:
      build_default_calls_strs.append('.{set_method_name}({value})'.format(
          set_method_name=set_method_name,
          value=param.default_value,
      ))

  get_methods_defs = '\n'.join(['\n  ' + s for s in get_methods_defs_strs])
  set_methods_defs = '\n'.join(['\n    ' + s for s in set_methods_defs_strs])

  build_default_calls = ''.join(['\n    ' + s for s in build_default_calls_strs])

  print(AUTO_VALUE_CLASS_TEMPLATE.format(
      autovalue_class_name=autovalue_class_name,
      func_name=func_name,
      get_methods_defs=get_methods_defs,
      set_methods_defs=set_methods_defs,
      build_default_calls=build_default_calls,
  ))


def upperfirst(s):
  return s[0].upper() + s[1:]


def replace_content_for_test_file(autovalue_class_name, func_name, params, content):
  func_name_anchor = '.' + func_name + '('
  output = ""
  while func_name_anchor in content:
    # E.g. "abcd blah.func(f1, f2, f3) hello"
    # is splitted to "abcd blah" and "f1, f2, f3) hello"
    pre, post = content.split(func_name_anchor, 1)

    # E.g. "f1, f2, f3"
    input_args_str = get_until_close_paren(post)
    replaced_input_args = replace_test_input_args_with_autovalue_class(autovalue_class_name, input_args_str, params)
    output += pre + func_name_anchor + replaced_input_args
    content = post[len(input_args_str):]

  # need to append the remaining content which does not have the func name
  # anchor
  output += content

  return output


def replace_content_for_call_site_file(autovalue_class_name, func_name, params, content):
  func_name_anchor = '.' + func_name + '('
  output = ""
  while func_name_anchor in content:
    # E.g. "abcd blah.func(f1, f2, f3) hello"
    # is splitted to "abcd blah" and "f1, f2, f3) hello"
    pre, post = content.split(func_name_anchor, 1)

    # E.g. "f1, f2, f3"
    input_args_str = get_until_close_paren(post)
    replaced_input_args = replace_input_args_with_autovalue_class(autovalue_class_name, input_args_str, params)
    output += pre + func_name_anchor + replaced_input_args
    content = post[len(input_args_str):]

  # need to append the remaining content which does not have the func name
  # anchor
  output += content

  return output


def replace_test_input_args_with_autovalue_class(autovalue_class_name, input_args_str, params):
  """
  Replaces input arguments string with an object of the autovalue class.
  Because this is for the test file, we assume "false" and "Optional.empty()"
  are not significant.

  E.g. "false, 3, Optional.empty(), "abc", true" -> "Params.builder()
    .setF2(3)
    .setF4("abc")
    .setF5(true)
    .build()
  """
  # E.g. "12, /* comment= */ 34, 45" -> ["12", "34", "45"]
  input_arg_strs = parse_input_arg_strs(input_args_str)
  output = autovalue_class_name + ".builder()"
  cleaned_arg_strs = [clean_input_arg(s) for s in input_arg_strs]

  for arg, param in zip(cleaned_arg_strs, params):
    if can_skip_arg(arg):
      continue
    output += "\n.set%s(%s)" % (upperfirst(param.name), arg)
  output += "\n.build()"
  return output


def replace_input_args_with_autovalue_class(autovalue_class_name, input_args_str, params):
  """
  Replaces input arguments string with an object of the autovalue class.

  E.g. "false, 3, Optional.empty(), "abc", true" -> "Params.builder()
    .setF1(false)
    .setF2(3)
    .setF3(Optional.empty())
    .setF4("abc")
    .setF5(true)
    .build()
  """
  # E.g. "12, /* comment= */ 34, 45" -> ["12", "34", "45"]
  input_arg_strs = parse_input_arg_strs(input_args_str)
  output = autovalue_class_name + ".builder()"
  cleaned_arg_strs = [clean_input_arg(s) for s in input_arg_strs]

  for arg, param in zip(cleaned_arg_strs, params):
    output += "\n.set%s(%s)" % (upperfirst(param.name), arg)
  output += "\n.build()"
  return output


def parse_input_arg_strs(s):
  """
  Parses the input values out from a string.

  E.g. "12, /* comment= */ 34, func(8,9)" ->
    ["12", "/* comment= */ 34", "func(8,9)"]
  """
  input_arg_strs = []
  sofar = []
  num_open_parens = 0
  for c in s:
    if c == ',' and not num_open_parens:
      input_arg_strs.append(''.join(sofar))
      sofar.clear()
      continue

    sofar.append(c)
    if c == '(':
      num_open_parens += 1
    if c == ')':
      num_open_parens -= 1

  if sofar:
    input_arg_strs.append(''.join(sofar))

  return input_arg_strs


def clean_input_arg(s):
  # strip out comments
  if "*/" in s:
    s = s.split("*/", 1)[1]

  return s.strip()


def can_skip_arg(arg):
  return arg in ("false", "Optional.empty()")


def transform_func_def(autovalue_class_name, func_name, params, func_def_str):
  """
  Transforms a function defintion with the the use of an auto value class.

  It does the following:
    1) Replace the input parameters with the auto value class alone.
    2) Replace all instances of the input parameters with the fetching
       of the corresponding fields from the params object.

  For example, given the following function definition:

  String convertFunc(type1 f1, type2 f2) {
    return f1 + f1 + f2;
  }

  It becomes (assuming the autovalue class name is Params):

  String convertFunc(Params params) {
    return params.f1() + params.f1() + params.f2();
  }
  """
  output = ""
  # E.g. If the auto value class name is InputParams, the params_var_name
  # should be inputParams.
  params_var_name = autovalue_class_name[0].lower() + autovalue_class_name[1:]

  # The example is splitted into:
  # "String convertFunc" and "type1 f1, type2 f2) {..."
  pre, post_open_paren_str = func_def_str.split('(', 1)
  output += pre
  output += "(%s %s)" % (autovalue_class_name, params_var_name)

  # params_str is "type1 f1, type2"
  params_str = get_until_close_paren(post_open_paren_str)
  # Skipped the params_str.
  # post_input_params_str is " {
  #   return f1 + f1 + f2;
  # }"
  post_input_params_str = post_open_paren_str[len(params_str) + 1:]

  for param in params:
      # Replaces all param name with a fetch of a field from the
      # autovalue class. E.g. "f1 + f2" -> "params.f1() + params.f2()".
      post_input_params_str = re.sub(
        r"\b%s\b" % param.name,
        "%s.%s()" % (params_var_name, param.name),
        post_input_params_str,
      )

  output += post_input_params_str
  return output


def parse_args(cmd_args):
  description = __doc__.split("Copyright 2018")[0].strip()

  parser = argparse.ArgumentParser(
    description=description,
    formatter_class=argparse.RawDescriptionHelpFormatter,
  )
  parser.add_argument("func_def_input_file", help="A file with the function definition")
  parser.add_argument("--autovalue_class_name", default=AUTO_VALUE_CLASS_NAME)
  parser.add_argument(
    "--output_autovalue_class",
    action="store_true",
    help="Output the definition of the autovalue class.")
  parser.add_argument(
    "--convert_method_calls_in_test_file",
    help="Convert method calls in a test file to use the autovalue class.")
  parser.add_argument(
	"--convert_call_site_file",
    help="Convert the original call invocation to use the autovalue class.")
  parser.add_argument(
    "--transform_func_def_with_autovalue", action="store_true",
    help="Transform the function definition by using the autovalue class.")

  args = parser.parse_args(cmd_args)
  return args


def main():
  args = parse_args(sys.argv[1:])

  app = App(args)
  sys.exit(app.run())


if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""sorno_realdate.py prints the human readable date for timestamps

Example:

    $ sorno_realdate.py 1455223642 1455223642000 1455223642000000 1455223642000000000
    1455223642: 2016-02-11 12:47:22-0800 PST in s
    1455223642000: 2016-02-11 12:47:22-0800 PST in ms
    1455223642000000: 2016-02-11 12:47:22-0800 PST in us
    1455223642000000000: 2016-02-11 12:47:22-0800 PST in ns

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import sys

from sornobase import datetimeutil


_datetime_format = "%Y-%m-%d %H:%M:%S%z %Z"


class RealDateApp(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        for n in self.args.numbers:
            try:
                n = int(n)
                dt, unit = datetimeutil.number_to_local_datetime(n)
                print(
                    "%s: %s in %s" % (
                        n,
                        dt.strftime(_datetime_format),
                        unit,
                    )
                )
            except ValueError:
                try:
                    print(
                        "%s: %s" % (
                            n,
                            datetimeutil.guess_local_datetime(n).strftime(
                                _datetime_format
                            ),
                        )
                    )
                except:
                    print("%s: invalid datetime" % n)


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "numbers",
        metavar="number",
        nargs="+",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = RealDateApp(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Prints out a random fact for fun

Example:

    Only one way to run the script now to get a fact:

        sorno_facts.py


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

from html.parser import HTMLParser

import logging

import sys
import subprocess

import requests

_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


class App(object):
    FACTS_URL = "http://randomfunfacts.com"

    def __init__(
        self,
    ):
        pass

    def run(self):
        resp = requests.get(self.FACTS_URL)

        parser = RandomFunFactsParser()

        parser.feed(resp.text)

        _PLAIN_LOGGER.info(parser.facts)


class RandomFunFactsParser(HTMLParser):
    """
    Parse out the facts in <strong><i>facts</i></strong>
    """
    def __init__(self):
        HTMLParser.__init__(self)

        self.in_strong = False
        self.in_facts = False
        self.facts = ""

    def handle_starttag(self, tag, attrs):
        if tag == "strong" and not self.in_strong:
            self.in_strong = True

        if tag == "i" and self.in_strong and not self.in_facts:
            self.in_facts = True

    def handle_endtag(self, tag):
        if tag == "strong":
            self.in_strong = False

        if tag == "i" and self.in_facts:
            self.in_facts = False

    def handle_data(self, data):
        if self.in_facts:
            self.facts += data

#
# logger-related functions
#

def setup_logger(
    logger,
    debug=False,
    stdout=False,
    log_to_file=None,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
):
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
    )
    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)

    if log_to_file is not None:
        init_command = 'mkdir -p %s' % os.path.dirname(log_to_file)

        subprocess.check_call(init_command, shell=True)

        hdlr = TimedRotatingFileHandler(
            log_to_file,
            when="midnight",
            interval=1,
        )
        hdlr.setFormatter(formatter)
        logger.addHandler(hdlr)


def create_plain_logger(logger_name, stdout=True):
    plain_logger = logging.getLogger(logger_name)
    plain_logger.propagate = False
    plain_logger.setLevel(logging.INFO)

    if stdout:
        out = sys.stdout
    else:
        out = sys.stderr

    handler = logging.StreamHandler(stream=out)
    handler.setFormatter(
        logging.Formatter(
            fmt="%(message)s",
            datefmt="%Y",  # does not matter
        )
    )

    plain_logger.addHandler(handler)
    return plain_logger


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(formatter=None, stdout=False):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)
    return hdlr

#
# end of logger-related functions
#

def parse_args(cmd_args):
    description = __doc__.split('Copyright 2014')[0].strip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = create_plain_logger("PLAIN")

    app = App(
    )
    app.run()


if __name__ == '__main__':
    main()
